unit ActionTreeView;

interface
{$I Directives.Inc}
uses SysUtils, Windows, Messages, Classes, Graphics, ImgList,
  {$IFDEF DELPHI_6_UP}RTLConsts,{$ELSE}Consts,{$ENDIF}
  Controls, ActnList, Commctrl, ComCtrls, SpeedButtonCep, BitOp,
  {$IFDEF DELPHI_6_UP} ActnMan, {$ENDIF}
  forms;

resourcestring
  StrECreateNode = '     TActionNode';

type
  TNodeClient = class; //        
  TNodeClients = class; //       
  TActionNode = class; //       
  TActionTreeView = class; //  ,   ,  

  TNodeClientClass = class of TNodeClient;
  TNodeClientsClass = class of TNodeClients;
  TActionNodeClass = class of TActionNode;


  //        
  TNodeClient = class(TCollectionItem)
  private
    fActionLink: TActionLink;
    fParentItem: TNodeClient;
    fItems: TNodeClients;
    fActionNode: TActionNode;
    fVisible: boolean;
    fEnabled: boolean;
    fChecked: boolean;
    fCaption: TCaption;
    fImageIndex: TImageIndex;
    fExpanded: boolean;
    fActionTreeView: TActionTreeView;
    procedure SetItems(const Value: TNodeClients);
    function GetActionTreeView: TActionTreeView;
    procedure SetVisible(const Value: boolean);
    function GetAction: TCustomAction;
    procedure SetAction(const Value: TCustomAction);
    procedure SetChecked(const Value: boolean);
    procedure SetEnabled(const Value: boolean);
    procedure Invalidate;
    procedure SetCaption(const Value: TCaption);
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetExpanded(const Value: boolean);
    procedure SetDefaultValues;
    function VisibleStored: Boolean;
    function EnabledStored: Boolean;
    function CheckedStored: Boolean;
    function ImageIndexStored: Boolean;
  protected
    function GetDisplayName: string; override;
    function NeedUpdate: boolean;
    procedure AssignValues(Source: TPersistent); virtual;
    class procedure AssignError(Source: TPersistent; ClassName: string);
    {$IFDEF DELPHI_2009_UP}
    procedure SetCollection(Value: TCollection); override;
    {$ENDIF}
    //  ItemsClass     Items.
    //  Nil,    TNodeClients
    function ItemsClass: TNodeClientsClass; virtual;
    //  ItemClass      Items
    //  Nil,    TNodeClient
    function ItemClass: TNodeClientClass; virtual;
    procedure DoUpdateNode; virtual;
    function CaptionStored: boolean;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property ActionNode: TActionNode read fActionNode;
    property ActionTreeView: TActionTreeView read fActionTreeView;
    //   True,         
    function RootVisible: boolean;
    procedure UpdateNode;
    property ParentItem: TNodeClient read fParentItem;
    procedure Assign(Source: TPersistent); override;
  published
    property Items: TNodeClients read fItems write SetItems;
    property Visible: boolean read fVisible write SetVisible stored VisibleStored;
    property Enabled: boolean read fEnabled write SetEnabled stored EnabledStored;
    property Checked: boolean read fChecked write SetChecked stored CheckedStored;
    property Action: TCustomAction read GetAction write SetAction;
    property Caption: TCaption read fCaption write SetCaption stored CaptionStored;
    property Expanded: boolean read fExpanded write SetExpanded default true;
    property ImageIndex: TImageIndex read fImageIndex write SetImageIndex stored ImageIndexStored;
  end;

  TIterateProc = procedure(Item: TNodeClient) of object;

  //       
  TNodeClients = class(TOwnedCollection)
  private
    function GetItem(Index: Integer): TNodeClient;
    procedure SetItem(Index: Integer; const Value: TNodeClient);
  protected
    procedure Update(Item: TCollectionItem); override;
    procedure UpdateNode(Item: TNodeClient);
  public
    constructor Create(AOwner: TPersistent; ItemClass: TNodeClientClass);
    procedure Assign(Source: TPersistent); override;
    class function GetActionTreeView(NodeClients: TNodeClients): TActionTreeView;
    class procedure IterateClients(NodeClients: TNodeClients; ActionProc: TIterateProc);
    property Items[Index: Integer]: TNodeClient read GetItem write SetItem; default;
  end;

  TMarkedNodeType = (mntNone, mntBefore, mntAfter, mntChild, mntRoot);

  //       
  TActionNode = class(TTreeNode)
  private
    fNodeClient: TNodeClient;
    function GetAction: TCustomAction;
  protected
    procedure DrawMarked(TreeView: TActionTreeView; R: TRect); virtual;
    procedure DrawImage(TreeView: TActionTreeView; State: TCustomDrawState;
      Image: TCustomImageList; Index: integer; R: TRect); virtual;
    procedure DoDraw(Canvas: TCanvas; TreeView: TActionTreeView; State: TCustomDrawState; var PaintImages, DefaultDraw:
      boolean); virtual;
  public
    constructor Create(AOwner: TTreeNodes); {$IFDEF DELPHI_2009_UP} override; {$ENDIF}
    destructor Destroy; override;
    function RootEnabled: boolean;
    property NodeClient: TNodeClient read fNodeClient;
    property Action: TCustomAction read GetAction;
  end;

  TGliphState = class(TCustomGliphs)
  private
    fChecked: TImageIndex;
    fUnchecked: TImageIndex;
    function GetActionTreeView: TActionTreeView;
    function GetChecked: TImageIndex;
    function GetUnchecked: TImageIndex;
    procedure SetChecked(const Value: TImageIndex);
    procedure SetUnchecked(const Value: TImageIndex);
  protected
    procedure Changed; override;
    function GetHandle: THandle; override;
  public
    constructor Create(AOwner: TActionTreeView);
    property ActionTreeView: TActionTreeView read GetActionTreeView;
    procedure Assign(Source: TPersistent); override;
  published
    property Checked: TImageIndex read GetChecked write SetChecked default 2;
    property Unchecked: TImageIndex read GetUnchecked write SetUnchecked default 1;
  end;

  //  ,   ,  
  TActionTreeView = class(TCustomTreeView)
  private
    fNodeClients: TNodeClients;
    fInCreateNode: boolean;
    fInCreateNodeClient: boolean;
    {$IFDEF DELPHI_2005_UP}
    fNeedFreeEmptyNodes: boolean;
    {$ENDIF}
    fBtnFrameColor: TColor;
    fSelectedColor: TColor;
    fDisabledFontColor: TColor;
    fStateIndex: array[boolean] of TImageIndex;
    fHighlightTextColor: TColor;
    fDragged: boolean;
    fDragNode: TActionNode;
    fTargetNode: TActionNode;
    fMarkedChange: boolean;
    fMarkedNodeType: TMarkedNodeType;
    fUpdateCount: integer;
    fGliphState: TGliphState;
    fIsDesigning: boolean;
    fRichImageList: TRichImageList;
    FImageChangeLink: TChangeLink;
    FStateChangeLink: TChangeLink;
    procedure SetNodeClients(const Value: TNodeClients);
    procedure SetBtnFrameColor(const Value: TColor);
    procedure SetSelectedColor(const Value: TColor);
    procedure SetDisabledFontColor(const Value: TColor);
    function GetStateIndex(Enabled: boolean): TImageIndex;
    procedure SetStateIndex(Enabled: boolean; const Value: TImageIndex);
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
    procedure SetHighlightTextColor(const Value: TColor);
    function GetColorDef(Color, DefaultColor: TColor): TColor;
    procedure CMDrag(var Message: TCMDrag); message CM_DRAG;
    procedure SetDragged(const Value: boolean);
    procedure SetTargetNode(const Value: TActionNode);
    procedure SetMarkedNodeType(const Value: TMarkedNodeType);
    {$IFDEF DELPHI_2005_UP}
    procedure WMSETREDRAW(var Message: TWMSETREDRAW); message WM_SETREDRAW;
    {$ENDIF}
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure SetGliphState(const Value: TGliphState);
  protected
    function ActionNodeClass: TActionNodeClass; virtual;
    function NodeClientsClass: TNodeClientsClass; virtual;
    function NodeClientsItemClass: TNodeClientClass; virtual;
    function CreateRichImageList: TRichImageList; virtual;
    property InCreateNode: boolean read fInCreateNode;
    property InCreateNodeClient: boolean read fInCreateNodeClient;
    function CreateNode: TTreeNode; override;
    {$IFDEF DELPHI_6_UP}
    function CreateNodes: TTreeNodes; override;
    {$ENDIF}
    procedure ImageListChange(Sender: TObject); virtual;
    procedure SetImages(Value: TCustomImageList); virtual;
    procedure SetStateImages(Value: TCustomImageList); virtual;
    procedure UpdateCanvasColor(Canvas: TCanvas; State: TCustomDrawState);
    procedure Expand(Node: TTreeNode); override;
    procedure Collapse(Node: TTreeNode); override;
    procedure BeginUpdate;
    procedure EndUpdate;
    property UpdateCount: integer read fUpdateCount;
    procedure KeyPress(var Key: Char); override;
    procedure DblClick; override;

    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure DoDragOver(Source: TDragObject; X, Y: Integer; CanDrop: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property StateIndex[Enabled: boolean]: TImageIndex read GetStateIndex write SetStateIndex;
    function CopyNodeClient(Sourc: TPersistent; Dest: TNodeClient;
      MarkedNodeType: TMarkedNodeType; DelSource: boolean = false): TNodeClient;
    function ExecuteSelected: boolean;
    procedure DropSelectedNode(DelChilds: boolean);

    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    property Dragged: boolean read fDragged write SetDragged;
    property DragNode: TActionNode read fDragNode;
    property TargetNode: TActionNode read fTargetNode write SetTargetNode;
    property MarkedNodeType: TMarkedNodeType read fMarkedNodeType write SetMarkedNodeType;
    property IsDesigning: boolean read fIsDesigning write fIsDesigning;
  published
    property NodeClients: TNodeClients read fNodeClients write SetNodeClients;
    property BtnFrameColor: TColor read fBtnFrameColor write SetBtnFrameColor default $00C66931;
    property SelectedColor: TColor read fSelectedColor write SetSelectedColor default $00EFD3C6;
    property HighlightTextColor: TColor read fHighlightTextColor write SetHighlightTextColor default clNone;
    property DisabledFontColor: TColor read fDisabledFontColor write SetDisabledFontColor default clGrayText;
    property GliphState: TGliphState read fGliphState write SetGliphState;

    property Align;
    property Anchors;
    property AutoExpand;
    property BiDiMode;
    property BorderStyle;
    property BorderWidth;
    property ChangeDelay;
    property Color;
    property Ctl3D;
    property Constraints;
    property DragKind;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property HotTrack;
    property Images write SetImages;
    property Indent;
    property ParentBiDiMode;
    property ParentColor default False;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RightClickSelect;
    property RowSelect;
    property ShowButtons;
    property ShowHint;
    property ShowLines;
    property ShowRoot;
    property StateImages write SetStateImages;
    property TabOrder;
    property TabStop default True;
    property ToolTips;
    property Visible;

    property OnChange;
    property OnChanging;
    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnExpanding;
    property OnExpanded;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation
...
