unit AlignBandsUtl;
(*
 * Module:          
 *              
 * Version:    1.0 (14.11.2007) b
 * Author:      
 * Comments:         .
 *                    
 *              : roschinspb@mail.ru
 *             http://www.delphikingdom.com/asp/users.asp?ID=1271
 *             http://www.roschinspb.boom.ru
 *
 * Copyright:   Cep Po 2007 .
 *)

interface
uses Windows, Types, Math, RTLConsts, SysUtils, Classes, Controls, ActnMan;

resourcestring
  ErrTBandRect = ' TBandRect (%d)';
  ErrInputParam = '  ';
  ErrId = ' %d  ';
  ErrNoId = '  %d';

const PageSize = 16;
      RealignCount = 3;
type
  TBandRect = record
    SourceRect, ResultRect: TRect;
    Row, RowCount: integer;
    MinW, MinH: integer;
    Id: integer;
    Fx, Fy, Vx, Vy, X, Y: Double;
  end;
  PBandRect = ^TBandRect;
  TBandRects = array [0..PageSize - 1] of TBandRect;
  PBandRects = ^TBandRects;

  TEnableReg = record
    Left, Width: integer;
    Index, Count: integer;
  end;
  TEnableRegs = array [0..15] of TEnableReg;
  PEnableRegs = ^TEnableRegs;

  TBandList = class (TObject)
  private
    fArray:PBandRects;
    fRowSize: integer;
    fMinRow: integer;
    fBufCount: integer;
    fCount: integer;
    fRighToLeft: boolean;
    fMaxRow: integer;
    fBoundsRect: TRect;
    fUpdateCount: integer;
    fonAligned: TNotifyEvent;
    fAlligning: boolean;
    procedure SetRowSize(const Value: integer);
    procedure UpdateRect(var BandRect: TBandRect);
    function GetBands(I: integer): TBandRect;
    procedure SetBands(I: integer; const Value: TBandRect);
    procedure CalcMinRow;
    function FirstLastIndex(Row, MinLeft, MaxRight: integer; var First, Last: integer): boolean;
    procedure SetRighToLeft(const Value: boolean);
    procedure UpdateSize(ARect: TRect);
  protected
    procedure InternalAdd(const BandRect: TBandRect;
                          const AArray:PBandRects;
                          var ACount: integer;
                          const ABufCount: integer);
    procedure AdjustRow(Row: integer; Left, Right: integer; MinLeft: integer = Low(Integer); MaxRight: integer = High(Integer));
    procedure Sort;
    procedure AdjustAll(ARect: TRect);
  public
    function CalcEnableRegs(Row: integer; EnableReg: TEnableReg;
      var EnableRegs: PEnableRegs): integer;
    constructor Create;
    destructor Destroy; override;
    property Alligning: boolean read fAlligning;
    property Count: integer read fCount;
    procedure Add(BandRect: TBandRect); overload;
    procedure Add(const ARect: TRect; const MinWidth, MinHeight, Id: integer); overload;
    procedure Delete(Index: integer);
    procedure SetSourceRect(I: integer);
    procedure Reset;
    procedure Init;
    procedure Realign(var ARect: TRect; const N: byte = RealignCount);
    property BoundsRect: TRect read fBoundsRect;
    property MinRow: integer read fMinRow;
    property MaxRow: integer read fMaxRow;
    property RowSize: integer read fRowSize write SetRowSize;
    property Bands[I: integer]: TBandRect read GetBands write SetBands; default;
    property RighToLeft: boolean read fRighToLeft write SetRighToLeft;
    function FindBand(Id: integer): integer;
    function BandById(Id: integer): TBandRect;
    procedure ChangeId(OldId, NewId: integer);
    procedure BeginUpdate;
    procedure EndUpdate;

    property onAligned: TNotifyEvent read fonAligned write fonAligned;
  end;

  TBandItems = class;

  TBandItem = class (TCollectionItem)
  private
    fUpdateCount:integer;
    fChanged:boolean;
    fChangedAll:boolean;
    fVisible: boolean;
    fBoundsRect: TRect;
    fInsets: TRect;
    fSysArea: Byte;
    fControl: TControl;
    fMinHeight: Word;
    fMinWidth: Word;
    fDestroy: boolean;
    procedure SetBoundsRect(const Value: TRect);
    function GetClientRect: TRect;
    procedure SetClientRect(const Value: TRect);
    procedure SetMinHeight(const Value: Word);
    procedure SetMinWidth(const Value: Word);
    procedure SetInsets(const Value: TRect);
    procedure SetSysArea(const Value: Byte);

    procedure SetVisible(const Value: boolean);
    procedure SetControl(const Value: TControl);
    function GetBandItems: TBandItems;
    procedure CalcConstraints(var MinW: Integer; var MinH: Integer);
    function GetSysRect: TRect;
    function GetHandRect: TRect;
  protected
    procedure Changed(AllItems: Boolean);
    procedure SetCollection(Value: TCollection); override;
    procedure ControlChanged; virtual;
  public
    procedure BeginUpdate;
    procedure EndUpdate;

    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property BandItems: TBandItems read GetBandItems;

    function IsVert: boolean;
    function IsRTL: boolean;
    function IntInsets: TRect;
    property BoundsRect: TRect read fBoundsRect write SetBoundsRect;
    property ClientRect: TRect read GetClientRect write SetClientRect;
    property SysRect: TRect read GetSysRect;
    property HandRect: TRect read GetHandRect;
    property MinWidth:Word read fMinWidth write SetMinWidth;
    property MinHeight:Word read fMinHeight write SetMinHeight;
    property Insets: TRect read fInsets write SetInsets;
    property SysArea: Byte read fSysArea write SetSysArea;
    property Control: TControl read fControl write SetControl;
    property Visible: boolean read fVisible write SetVisible;
  end;

  TBandItems = class (TOwnedCollection)
  private
    fBandList: TBandList;
    fOrientation: TBarOrientation;
    fBoundsRect: TRect;
    procedure SetOrientation(const Value: TBarOrientation);
    procedure Aligned(Sender: TObject);
    function GetRowSize: integer;
    procedure SetRowSize(const Value: integer);
    function GetBoundsRect: TRect;
    procedure SetBoundsRect(const Value: TRect);
    function GetItem(Index: Integer): TBandItem;
    procedure IntAligned;
  protected
    function IsVert: boolean;
    function RevertRect(const R: TRect): TRect;
    procedure DoAligned; virtual;
    procedure DoPainted; virtual;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TPersistent; ItemClass: TCollectionItemClass);
    destructor Destroy; override;
    procedure BeginUpdate; override;
    procedure EndUpdate; override;
    function Add(Control: TControl): TBandItem;
    procedure Delete(Index: Integer);
    property BoundsRect: TRect read GetBoundsRect write SetBoundsRect;
    property BandList: TBandList read fBandList;
    function BandByControl(Control: TControl):TBandItem;
    property RowSize: integer read GetRowSize write SetRowSize default 24;
    property Items[Index: Integer]: TBandItem read GetItem; default;
    property Orientation: TBarOrientation read fOrientation write SetOrientation;
  end;

implementation
...
