unit SpeedButtonCep;
(*
 * Module:            TSpeedButtonCep
 *                      Delphi
 *                 
 *             ControlsCepEx.dpk  EditorsCepEx.dpk.
 * Version:    2.8.0.0 (20.01.2010)
 * Author:      
 * Comments:         .
 *                    
 *              : roschinspb@mail.ru
 *             http://www.delphikingdom.com/asp/users.asp?ID=1271
 *             http://www.roschinspb.narod.ru
 *
 * Copyright:   2006 - 2010 .
 *
 * Version:    2.8.0.0 (20.01.2010)
 * Realized:     TCustomGliphs
 *
 * Version:    2.2 (21.09.2008)
 * Realized:     Delphi 2009.
 *
 * Version:    2.1 (18.09.2007)
 * Realized:      XP. .  ThemeEnabled.
 *)
interface
{$I Directives.Inc}
uses
  Windows, Messages, SysUtils, Classes,
  {$IFDEF DELPHI_6_UP}
  RTLConsts,
  {$ELSE}
  Consts,
  {$ENDIF}
  Controls, ImgList, Graphics,
  {$IFDEF VER130}{$ELSE}GraphUtil,{$ENDIF}
  GraphUtilEx, CommCtrl, ActnList, ExtCtrls, Buttons, BitOp, InternalTimer;

resourcestring
  NoAssign = '   %s  %s';
  NoIndexImg = '   (%d)';
  NoAssignfList = '  fList';

type
  TCepButtonKind = (CepBKImage, CepBKNone,
    CepBKLeftChevron, CepBKRightChevron, CepBKUpChevron, CepBKDownChevron,
    CepBKLeftArrow, CepBKRightArrow, CepBKUpArrow, CepBKDownArrow,
    CepBkCheck, CepBkCheckShadow,
    CepBKBulets);
  TInvalidateMode = (imInvalidate, imRepaint, imNone);
  TCepButtonStyle = (cbsNormal, cbsDropDown, cbsDropRight, cbsDrop);

  TCustomGliphs = class(TPersistent)
  private
    fOwner: TComponent;
    fOnChanged: TNotifyEvent;
  protected
    procedure Changed; virtual;
    function CheckImageIndex(I: integer): boolean; virtual;
    function GetHandle: THandle; virtual;
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TComponent);
    procedure LoadFromStream(S: TStream); virtual;
    procedure SaveToStream(S: TStream); virtual;
    property Handle: THandle read GetHandle;
    property Owner: TComponent read fOwner;
    property OnChanged: TNotifyEvent read fOnChanged write fOnChanged;
  end;

  TGliphsClass = class of TCustomGliphs;

  TGliphs = class(TCustomGliphs)
  private
    fImageIndexNormal: TImageIndex;
    fImageIndexPush: TImageIndex;
    fImageIndexCheck: TImageIndex;
    fImageIndexNormalHot: TImageIndex;
    fImageIndexCheckHot: TImageIndex;
    fImageIndexDisable: TImageIndex;
    fImageIndexCheckDisable: TImageIndex;
    procedure SetImageIndexCheck(const Value: TImageIndex);
    procedure SetImageIndexCheckDisable(const Value: TImageIndex);
    procedure SetImageIndexCheckHot(const Value: TImageIndex);
    procedure SetImageIndexDisable(const Value: TImageIndex);
    procedure SetImageIndexNormal(const Value: TImageIndex);
    procedure SetImageIndexPush(const Value: TImageIndex);
    procedure SetIndexNormalHot(const Value: TImageIndex);
  protected
    procedure Changed; override;
  public
    constructor Create(AOwner: TControl);
    procedure Assign(Source: TPersistent); override;
  published
    property ImageIndexNormal: TImageIndex read fImageIndexNormal write SetImageIndexNormal default -1;
    property ImageIndexPush: TImageIndex read fImageIndexPush write SetImageIndexPush default -1;
    property ImageIndexCheck: TImageIndex read fImageIndexCheck write SetImageIndexCheck default -1;
    property ImageIndexNormalHot: TImageIndex read fImageIndexNormalHot write SetIndexNormalHot default -1;
    property ImageIndexCheckHot: TImageIndex read fImageIndexCheckHot write SetImageIndexCheckHot default -1;
    property ImageIndexDisable: TImageIndex read fImageIndexDisable write SetImageIndexDisable default -1;
    property ImageIndexCheckDisable: TImageIndex read fImageIndexCheckDisable write SetImageIndexCheckDisable default -1;
    property OnChanged;
  end;

  TSpeedButtonCep = class(TGraphicControl)
  private
    fGliphs: TGliphs;
    fList: TRichImageList;
    FImageChangeLink: TChangeLink;
    fAutoCheck: boolean;
    fTimer: TTimer;
    fDefaultIndex: boolean;
    fClicks: TNotifyEvent;
    fIsClicks: boolean;
    fButtonKind: TCepButtonKind;
    fAutoHot: boolean;
    fHotChanged: TNotifyEvent;
    fCountUpdate: integer;
    fImageIndex: integer;
    fInvalidateMode: TInvalidateMode;
    fDropDown: TNotifyEvent;
    fStyle: TCepButtonStyle;
    fChildButton: TGraphicControl;
    fTimerHot: TInternalTimer;
    fInvalidating: boolean;
    procedure HotProc(Sender: TObject); //      
    procedure AddToEdit(Control: TWinControl);
    procedure SbbFromEdit(Control: TWinControl);
    procedure SetImageList(const Value: TCustomImageList);
    procedure ImageListChange(Sender: TObject);
    procedure ListChange(Sender: TObject);
    procedure SetChecked(const Value: boolean);
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure CMVISIBLECHANGED(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure CMENABLEDCHANGED(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LButtonUp;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure WMSETTEXT(var Message: TWMSETTEXT); message WM_SETTEXT;
    procedure WMGETTEXTLENGTH(var Message: TWMGETTEXTLENGTH); message WM_GETTEXTLENGTH;
    procedure WMGETTEXT(var Message: TWMGETTEXT); message WM_GETTEXT;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure SetBorder(const Value: TCepButtonBorder);
    procedure SetBtnFrameColor(const Value: TColor);
    procedure SetSelectedColor(const Value: TColor);
    procedure SetTransparent(const Value: boolean);
    procedure SetImageIndex(const Value: integer);
    procedure SetPushed(const Value: boolean);
    procedure StartTimer;
    procedure StopTimer;
    procedure onTimer(Sender: TObject);
    procedure SetHot(const Value: boolean);
    procedure SetGliphs(const Value: TGliphs);
    procedure SetBlend(const Value: byte);
    procedure SetButtonKind(const Value: TCepButtonKind);
    procedure SetAutoHot(const Value: boolean);
    function GetBorder: TCepButtonBorder;
    function GetPushed: boolean;
    function GetBlend: byte;
    function GetBtnFrameColor: TColor;
    function GetSelectedColor: TColor;
    function GetImageList: TCustomImageList;
    function GetTransparent: boolean;
    function GetImageIndex: integer;
    function GetHot: boolean;
    function GetChecked: boolean;
    function GetLayout: TButtonLayout;
    function GetSpacing: byte;
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetSpacing(const Value: byte);
    procedure SetStyle(const Value: TCepButtonStyle);
    procedure ClickChild(Sender: TObject);
    procedure ChildHotChanged(Sender: TObject);
    procedure UpdateChildBlend;
    procedure DoResize;
    function GetThemeEnabled: boolean;
    procedure SetThemeEnabled(const Value: boolean);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure WndProc(var Message: TMessage); override;
    procedure SetFocus; dynamic;
    procedure KillFocus; dynamic;
    //       
    procedure PaintToCanvas(Canvas: TCanvas; X, Y: integer; Transparent: boolean);
    procedure Updated; override;
    procedure UpdateImageIndex;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure HotChanged; dynamic;
    procedure DropDown; dynamic;
    function CreateChildButton(Style: TCepButtonStyle): TGraphicControl; dynamic;
    procedure UpdateChildButton;
    procedure SetParent(AParent: TWinControl); override;
    procedure Loaded; override;
    procedure Resize; override;
    property List: TRichImageList read fList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ImageIndex: integer read GetImageIndex write SetImageIndex;
    //     True,   
    property Pushed: boolean read GetPushed write SetPushed;
    //     True,    
    //  
    property Hot: boolean read GetHot write SetHot;
    procedure Repaint; override;
    procedure Invalidate; override;
    procedure Paint; override;
    procedure Clicks; dynamic;
    procedure Click; override;
    property AutoHot: boolean read fAutoHot write SetAutoHot;
    procedure BeginUpdate;
    procedure EndUpdate;
    property InvalidateMode: TInvalidateMode read fInvalidateMode write fInvalidateMode;
  published
    property Action;
    property Anchors;
    property Align;
    //    ,    
    //    Checked
    property AutoCheck: boolean read fAutoCheck write fAutoCheck default false;
    //   
    property Border: TCepButtonBorder read GetBorder write SetBorder default CepSingleBorder;
    // . 255 -  ; 0 - 
    property Blend: byte read GetBlend write SetBlend default 255;
    //  ButtonKind ,     
    property ButtonKind: TCepButtonKind read fButtonKind write SetButtonKind default CepBKImage;
    property Caption;
    //   Checked ,     
    property Checked: boolean read GetChecked write SetChecked default false;
    property Color;
    //  
    property BtnFrameColor: TColor read GetBtnFrameColor write SetBtnFrameColor default $00C66931;
    //   / 
    property SelectedColor: TColor read GetSelectedColor write SetSelectedColor default $00EFD3C6;
    property Constraints;
    property Enabled;
    property Font;
    //      
    property Gliphs: TGliphs read fGliphs write SetGliphs;
    //  
    property ImageList: TCustomImageList read GetImageList write SetImageList;
    //    Transparent,     
    property Transparent: boolean read GetTransparent write SetTransparent default false;
    //  Layout        
    property Layout: TButtonLayout read GetLayout write SetLayout default blGlyphLeft;
    //     
    property Spacing: byte read GetSpacing write SetSpacing default 4;
    //       
    //  .
    property Style: TCepButtonStyle read fStyle write SetStyle default cbsNormal;
    //  ThemeEnabled   ,   CepSingleBorder, CepFlatBorder
    //    windows.  Transparent     
    property ThemeEnabled: boolean read GetThemeEnabled write SetThemeEnabled default false;
    property ShowHint;
    property ParentShowHint;
    property ParentColor;
    property ParentFont;
    property PopupMenu;
    property Visible;
    property OnClick;
    //    ,  ,    
    //  
    property onClicks: TNotifyEvent read fClicks write fClicks;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    //         , 
    //    
    property onHotChanged: TNotifyEvent read fHotChanged write fHotChanged;
    //        
    //  . .  Style
    property OnDropDown: TNotifyEvent read fDropDown write fDropDown;
  end;

procedure Register;

implementation
...
