unit TextUtlCep;

(*
 * Module:         
 * Version:    1.1.0.0 (24.09.2008)
 * Author:      
 * Comments:     ()  
 *             http://www.delphikingdom.com/asp/viewitem.asp?catalogid=722
 *             ,     
 *
 * Copyright:   2008 Cep Po
 *
 * Version:    1.1.0.0 (24.09.2008)
 * Realized:     Delphi 2009
 *                
 *)
interface
{$I Directives.inc}
uses Windows, Messages, Consts, SysUtils, Classes, Graphics, ComCtrls, RichEdit, ComStrs
{$IFDEF DELPHI_6_UP}, RTLConsts{$ELSE}, GraphUtilEx{$ENDIF}
{$IFDEF DELPHI_2009_UP}, Character{$ENDIF};

type
  TTokenRec = record
    StartWord, EndWord: WideString;
  end;
{$Z4}
  TCharsCollection = set of AnsiChar;

const PageSize = 10240 div 2;
  CommentToken1: TTokenRec = (StartWord: '{'; EndWord: '}');
  CommentToken2: TTokenRec = (StartWord: '(*'; EndWord: '*)');
  CommentToken3: TTokenRec = (StartWord: '//'; EndWord: #13#10);
  CharBackSpaces: TCharsCollection = [' ', #0, #9, #10, #13, #160];

  RTFformat: WideString = 'Rich Text Format';
  UM_PAINTLINE = WM_USER + 13;


resourcestring
  ResInvalidValue = '   ''%s''';

type
  TClipboardFormats = array[0..2] of integer;
  TRichEdit = class(ComCtrls.TRichEdit)
  private
    fRightLine: integer;
    fFirstLeftLine: integer;
    fLeftLine: integer;
    procedure SaveToStream(Stream: TStream; TextType: Integer);
    procedure LoadFromStream(Stream: TStream; TextType: Integer);
    function GetTextW: WideString;
    procedure SetTextW(const Value: WideString);
    procedure WMERASEBKGND(var Message: TWMERASEBKGND); message WM_ERASEBKGND;
    procedure UMPAINTLINE(var Message: TMessage); message UM_PAINTLINE;
    procedure SetRightLine(const Value: integer);
    procedure DrawLines(DC: HDC);
    procedure SetFirstLeftLine(const Value: integer);
    procedure SetLeftLine(const Value: integer);
  public
    function IsEnablePaste(var Formats: TClipboardFormats): integer;
    procedure CopyAll;
    procedure PasteAll;
    property TextW: WideString read GetTextW write SetTextW;
    property RightLine: integer read fRightLine write SetRightLine;
    property LeftLine: integer read fLeftLine write SetLeftLine;
    property FirstLeftLine: integer read fFirstLeftLine write SetFirstLeftLine;
  end;



  TTokenPositions = record
    Offset, StartPos, EndPos: LongInt;
  end;

  TFindedText = class
  private
    fWideText: PWideChar;
    fLen: LongInt;
    fTokenPositions: array of TTokenPositions;
    fMaxBufferSize: LongInt;
    fRightMargin: Word;
    fCodePage: LongInt;
    function InternalFind(const Offset: Integer; var StartPos, EndPos: LongInt;
      const Next: boolean): boolean;
  protected
    procedure SetWideText(const Value: PWideChar); virtual;
    //       
    //Offset   ( )   ,     
    //   0.      
    //  
    //       , 
    //   ,  -1.
    function DoFind(var Offset: LongInt): LongInt; virtual; abstract;
    function ScanWhileBS(var X: integer): boolean;
  public
    constructor Create(WideText: PWideChar); virtual;
    procedure Clear; virtual;
    property Len: LongInt read fLen;
    property WideText: PWideChar read fWideText write SetWideText;
    property RightMargin: Word read fRightMargin write fRightMargin default 80;
    property CodePage: LongInt read fCodePage write fCodePage default 1251;
    property MaxBufferSize: LongInt read fMaxBufferSize;
    //     .
    //     ,   
    //c     
    //Offset -  ( )   ,     
    //   0
    function FindNext(var Offset: LongInt): LongInt; virtual;
    // ,       
    // Offset  ,   True,  StartPos, EndPos 
    //     ,    
    //False,  StartPos  EndPos   .
    function InToken(const Offset: LongInt; var StartPos, EndPos: LongInt): boolean; virtual;
    //    
    //Offset - .    ,     
    //StartPos, EndPos -      
    //S -       
    //   True   .  False.    , 
    //tartPos, EndPos, S   .
    function GetStripText(const Offset: LongInt; var StartPos, EndPos: LongInt; var S: WideString): boolean; virtual; abstract;
    //    
    function GetTokenText(const StartPos, EndPos: LongInt; const S: WideString): WideString; virtual; abstract;
  end;

  TQuotedText = class(TFindedText)
  private
    fOwner: TQuotedText;
    fQuote: WideChar;
    fCommentTokens: array of TTokenRec;
    fEnableFastFind: boolean;
    fPrefix: WideChar;
    fHexPrefix: WideChar;
    procedure SetQuote(const Value: WideChar);
    function GetCommentTokens(Index: integer): TTokenRec;
    procedure SetCommentTokens(Index: integer; const Value: TTokenRec);
    function GetCountCommentTokens: integer;
    procedure UpdateEnableFastFind;
    procedure SetPrefix(const Value: WideChar);
    procedure SetHexPrefix(const Value: WideChar);
  protected
    function ExtractChar(var Offset: Integer; var W: Word): boolean; virtual;
    function CheckQuotedText(var Offset: Integer): boolean; virtual;
    function FindFirstToken(var X1, X2: Integer): Integer; virtual;
    function DoFind(var Offset: LongInt): LongInt; override;
    property Prefix: WideChar read fPrefix write SetPrefix;
    property HexPrefix: WideChar read fHexPrefix write SetHexPrefix;
    function IsSimbol(C: WideChar): boolean;
    function IsLetterOrDigit(C: WideChar): boolean;
    procedure NextCommentOrLiteral(var StartPos, EndPos: Integer);
    function NextSimbol(var StartPos: Integer; First: boolean = true): boolean;
    function NextWord(var StartPos, EndPos: Integer; First: boolean = true): boolean;
    function NextWordFromList(const WordList: TStringList; var StartPos, EndPos: Integer; First: boolean = true): Integer;
    function NextBrackets(const Open, Close: WideChar; var StartPos, EndPos: Integer; First: boolean = true): boolean;
  public
    constructor Create(WideText: PWideChar); overload; override;
    destructor Destroy; override;
    procedure Assign(Source: TQuotedText); virtual;
    constructor Create(AOwner: TQuotedText); reintroduce; overload; virtual;
    property Quote: WideChar read fQuote write SetQuote;
    property CommentTokens[Index: integer]: TTokenRec read GetCommentTokens write SetCommentTokens;
    procedure AddCommentTokens(Value: TTokenRec);
    property CountCommentTokens: integer read GetCountCommentTokens;
    function GetStripText(const Offset: LongInt; var StartPos, EndPos: LongInt; var S: WideString): boolean; override;
    function GetTokenText(const StartPos, EndPos: LongInt; const S: WideString): WideString; override;
  end;

  TCommentedText = class(TQuotedText)
  protected
    function DoFind(var Offset: LongInt): LongInt; override;
  public
    function GetStripText(const Offset: LongInt; var StartPos, EndPos: LongInt; var S: WideString): boolean; override;
    function GetTokenText(const StartPos, EndPos: LongInt; const S: WideString): WideString; override;
  end;

  TLiteralText = class(TQuotedText)
  private
    fConcateChar: WideString;
    fDefaultHex: boolean;
    fCommentList: TObject;
    fColumn: integer;
    procedure SetConcateChar(const Value: WideString);
    function ScanWhileBackSpace(var X: Integer;
      AcceptConcate: boolean): boolean;
  protected
    function DoFind(var Offset: LongInt): LongInt; override;
  public
    constructor Create(WideText: PWideChar); override;
    destructor Destroy; override;
    property Prefix default '#';
    property HexPrefix default '$';
    property DefaultHex: boolean read fDefaultHex write fDefaultHex;
    property ConcateChar: WideString read fConcateChar write SetConcateChar;
    function GetStripText(const Offset: LongInt; var StartPos, EndPos: LongInt; var S: WideString): boolean; override;
    function GetTokenText(const StartPos, EndPos: LongInt; const S: WideString): WideString; override;
    procedure UpdateComments(StartPos, EndPos: LongInt);
    property Column: integer read fColumn write fColumn;
    function MakeText(StripText: WideString;
      const Hex: boolean = false;
      const FirstLeft: Longint = 0;
      Left: Longint = 0;
      Right: Longint = 80;
      const CodePage: LongInt = 1251;
      const NoComments: boolean = true): WideString;
    function GetMargins(const StartPos, EndPos: Integer;
      var FirstLeft, Left, Right: integer): boolean;
    procedure TruncRight(var Result: WideString);
  end;

  TFunctionText = class(TQuotedText)
  private
    fList: TStringList;
    fObjName: WideString;
    fObjClass: WideString;
    fObjReturn: WideString;
    fClassEndPos: integer;
    fParams: TStringList;
    fParamTypes: TStringList;
    fObjKind: Integer;
  protected
    function DoFind(var Offset: LongInt): LongInt; override;
  public
    constructor Create(WideText: PWideChar); override;
    destructor Destroy; override;

    property ObjClass: WideString read fObjClass;
    property ObjName: WideString read fObjName;
    property ObjReturn: WideString read fObjReturn;
    property ObjKind: Integer read fObjKind;
    property ClassEndPos: integer read fClassEndPos;
    property Params: TStringList read fParams;
    property List: TStringList read fList;
    property ParamTypes: TStringList read fParamTypes;

  end;


function UpCaseW(Ch: WideChar): WideChar;
function HexToInt(s: AnsiString): Longword;

function FindFirstWord(const WText: PWideChar; var Ofs: LongInt; Quote: WideChar; Token0, Token1, Token2, Token3: DWORD; CharCount: LongInt): integer; overload;
function FindFirstWord(const WText: PWideChar; var Ofs: LongInt; Quote: WideChar; Token0, Token1, Token2, Token3: DWORD): integer; overload;

function FindEOS(const WText: PWideChar; var Ofs: LongInt; CharCount: LongInt): integer; overload;
function FindEOS(const WText: PWideChar; var Ofs: LongInt): integer; overload;

function FindQuotedText(Ch: WideChar; const S: PWideChar; var Offset: LongInt; CharCount: LongInt): LongInt; overload;
function FindQuotedText(Ch: WideChar; const S: PWideChar; var Offset: LongInt): LongInt; overload;

function PosStrEx(const SubStr: WideString; const S: PWideChar; var Offset: LongInt; CharCount: LongInt): LongInt; overload;
function PosStrEx(const SubStr: WideString; const S: PWideChar; var Offset: LongInt): LongInt; overload;
function PosStrIns(const SubStr: WideString; const S: PWideChar; var Offset: LongInt; CharCount: LongInt): LongInt;

function PosWord(const S: PWideChar; var Offset: LongInt; CharCount: LongInt): LongInt;


function WideCharToMultiByte(CodePage: LongInt; WC: Word; var C: AnsiChar): boolean;
//        .
//   ,    
function StringToList(const S: WideString; const Separator: WideChar; List: TStringList):
  integer;
function ListToString(List: TStringList; const Separator: WideChar): WideString;

procedure PosToColRow(const WText: PWideChar; const Ofs: LongInt; var Col, Row: integer; Len: LongInt = -1);

implementation
...
