unit UMultiThread;
(*
 * Module:     TMultiThreadFor      
 *               .
 * Version:    2.5 (14.11.2008)
 * Author:      
 * Comments:      :
 *             .   for Z := StartValueZ to FinishValueZ do
 *             .     for Y := StartValueY to FinishValueY do
 *             .       for X := StartValueX to FinishValueZ do
 *             .         Execute(X, Y, Z)
 *                 ().
 *             :
 *             1.      onExecute   ,
 *                   ,
 *               .
 *             2.  onStartThread, onFinishThread, onExecute 
 *               ,       
 *                 .
 *             3.  onExecute,       
 *             ,   TMultiThreadFor     
 *             ,    .
 *                   .
 *                    
 *              : roschinspb@mail.ru
 *             http://www.delphikingdom.com/asp/users.asp?ID=1271
 *             http://www.roschinspb.narod.ru
 *
 * Copyright:   2008
 *
 * Version:    2.5 (14.11.2008)
 * Realized:   
 *)
interface
{$I Directives.inc}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  InternalTimer;

resourcestring
  StrEStarted = '  ,     %s';
  StrENoStarted = '  ';
  StrEParam = '  (%s)  %s';

const
  OrderNames: array[0..2] of string = ('X', 'Y', 'Z');
  MaxPercent = 1000;             //   onPercent   0..MaxPercent
  DelayTerminate = 1000;         //       Stop
  DelayInTimer = 10;             //     
  TimerInterval = 40;            //    onPercent
  IndexNull = $FFFFFFFF;

type
  //  TMultiThreadFor
  TMultiThreadStatus = (mtsNull, //   
    mtsStarted,                  //  
    mtsTerminated,               //  
    mtsError,                    //     
    mtsSucceeds);                //   
  //   
  TOrderRec = record
    StartValue: Extended;        //  
    FinishValue: Extended;       //  
    Count: DWORD;                //  
  end;
  //      
  TErrorStack = record
    Exception: ExceptClass;
    Message: string;
    Index: DWORD;
    VectorIndex: array of DWORD;
    Vector: array of Extended;
  end;

  TMultiThreadFor = class;

  //    TMultiThreadFor
  EMultiThread = class(Exception);

  //  
  TMultiThreadForEvent = procedure(Sender: TMultiThreadFor; ThreadIndex: Word) of object;
  TMultiThreadForExecute = procedure(Sender: TMultiThreadFor;
                                     ThreadIndex: Word;
                                     const Index: DWORD;
                                     const VectorIndex: array of DWORD;
                                     const Vector: array of Extended) of object;
  TBeforeStart = procedure(Sender: TMultiThreadFor) of object;
  TAfterFinish = procedure(Sender: TMultiThreadFor; const ErrorStack: array of TErrorStack) of object;
  TonPercent = procedure(Sender: TMultiThreadFor; const Percent: integer) of object;

  //        
  TOrderItem = class(TCollectionItem)
  private
    fItem: TOrderRec;
    fDisplayName: string;
    function GetCount: DWORD;
    function GetFinishValue: Extended;
    function GetStartValue: Extended;
    procedure SetCount(const Value: DWORD);
    procedure SetFinishValue(const Value: Extended);
    procedure SetStartValue(const Value: Extended);
    procedure SetItem(const Value: TOrderRec);
    procedure CheckCollection(Collection: TCollection);
  protected
    {$IFDEF DELPHI_6_UP}
    procedure SetCollection(Value: TCollection); override;
    {$ENDIF}
    procedure SetIndex(Value: Integer); override;
    function GetDisplayName: string; override;
    procedure SetDisplayName(const Value: string); override;
  public
    constructor Create(Collection: TCollection); override;
    property Item: TOrderRec read fItem write SetItem;
    procedure Assign(Source: TPersistent); override;
    procedure LoadFromStream(S: TStream);
    procedure SaveToStream(S: TStream);
  published
    property StartValue: Extended read GetStartValue write SetStartValue;
    property FinishValue: Extended read GetFinishValue write SetFinishValue;
    property Count: DWORD read GetCount write SetCount default 1;
    property DisplayName;
  end;

  //    
  TOrderItems = class(TOwnedCollection)
  private
    function GetItem(Index: Integer): TOrderItem;
    procedure SetItem(Index: Integer; const Value: TOrderItem);
  protected
    procedure Update(Item: TCollectionItem); override;
  published
  public
    constructor Create(AOwner: TPersistent);
    property Items[Index: Integer]: TOrderItem read GetItem write SetItem; default;
  end;

  //  
  TMultiThreadFor = class(TComponent)
  private
    fForParams: array of TOrderRec;
    fDelta: array of Extended;
    fThreads: array of TThread;
    fIndexes: array of DWORD;
    fExecuteCount: DWORD;
    FLock: TRTLCriticalSection;
    fStarted: boolean;
    fDefThreadCount: Word;
    fThreadCount: Word;
    fonStartThread: TMultiThreadForEvent;
    fonFinishThread: TMultiThreadForEvent;
    fonExecute: TMultiThreadForExecute;
    fPartCount: DWORD;
    fOrders: TOrderItems;
    fErrorStack: array of TErrorStack;
    fonBeforeStart: TBeforeStart;
    fonAfterFinish: TAfterFinish;
    fonPercent: TonPercent;
    fLastPercent: integer;
    fTimer: TInternalTimer;
    fStatus: TMultiThreadStatus;
    procedure CheckStarted;
    procedure SetDefThreadCount(const Value: Word);
    procedure SetonStartThread(const Value: TMultiThreadForEvent);
    procedure SetonFinishThread(const Value: TMultiThreadForEvent);
    procedure SetonExecute(const Value: TMultiThreadForExecute);
    procedure SetPartCount(const Value: DWORD);
    procedure SetOrderItems(const Value: TOrderItems);
    function GetThreads(Index: integer): TThread;
    procedure DelCriticalSection;
    procedure SetonBeforeStart(const Value: TBeforeStart);
    procedure SetonAfterFinish(const Value: TAfterFinish);
    procedure DoTimer(Sender: TObject);
  protected
    procedure IndexToVector(Index: DWORD;
                        var VectorIndex: array of DWORD;
                        var Vector: array of Extended);
    procedure DoStartThread(ThreadIndex: Word); virtual;
    procedure DoFinishThread(ThreadIndex: Word); virtual;
    procedure DoExecute(const ThreadIndex: Word;
                        const Index: DWORD;
                        const VectorIndex: array of DWORD;
                        const Vector: array of Extended); virtual;
    procedure DoBeforeStart; virtual;
    procedure DoAfterFinish(const ErrorStack: array of TErrorStack); virtual;
    procedure DoPercent(const Percent: integer); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    // .        
    procedure Start;
    //    .
    procedure Stop;
    //   ,    Delay .
    function Wait(Delay: DWORD): DWORD;
    // Threads    ()
    property Threads[Index: integer]: TThread read GetThreads;
    // ThreadCount    
    property ThreadCount: Word read fThreadCount;
    //  Started = true,    
    property Started: boolean read fStarted;
    // Status     TMultiThreadFor
    property Status: TMultiThreadStatus read fStatus;
    //   
    procedure Lock;
    //   
    procedure UnLock;
  published
    //  DefThreadCount   ,   .
    //  0,    ,    .
    property DefThreadCount: Word read fDefThreadCount write SetDefThreadCount default 0;
    //  PartCount   ,  
    //   
    property PartCount: DWORD read fPartCount write SetPartCount default 1000;
    //   
    property Orders: TOrderItems read fOrders write SetOrderItems;

    //       
    //  onStartThread     
    property onStartThread: TMultiThreadForEvent read fonStartThread write SetonStartThread;
    //  onFinishThread     ,  
    //    ,    
    property onFinishThread: TMultiThreadForEvent read fonFinishThread write SetonFinishThread;
    //   onExecute      
    //   
    property onExecute: TMultiThreadForExecute read fonExecute write SetonExecute;

    //       .
    //   ,      .
    //  onBeforeStart     
    property onBeforeStart: TBeforeStart read fonBeforeStart write SetonBeforeStart;
    //  onAfterFinish   / 
    property onAfterFinish: TAfterFinish read fonAfterFinish write SetonAfterFinish;
    //  onPercent     
    //    .
    property onPercent: TonPercent read fonPercent write fonPercent;
  end;

procedure Register;

implementation
...
