unit WindowThread;
(*
 * Module:      TThreadWindow   TThread
 *                 ,   
 *              .
 *                 ,
 *                .
 * Version:    03.00 (16.04.2009)
 * Author:     Cep Po
 * Comments:      Delphi
 *              : http://www.roschinspb.narod.ru
 *              : http://www.delphikingdom.com/asp/users.asp?ID=1271
 *
 * Copyright:   2005 - 2009 .
 *
 * Version:    03.00 (16.04.2009)
 * Realized:       Message  (  Delphi 
 *              ).
 *             . http://www.delphikingdom.com/asp/viewitem.asp?catalogid=1390
 *               
 *)
interface
uses Windows,
  Messages,
  Classes,
  SysUtils,
  Math
  {$IFDEF VER130},
  OldVersions{$ELSE}, GraphUtil{$ENDIF}
  , GraphUtilEx
  ;
{$I Directives.inc}
const
  NameIconWait = 'MESICON9'; // ,     
  NameClass: string = 'Class Window of TThreadWindow (new)'; //  
  InterValTimeOut = 20000; //  ,    , 
  //     (0 -  )

resourcestring
  DefaultMessage = '   !';
  TextCancel = '';
  ErrorWait = '  ';
  ErrorThread = '      %1:s' + #13#10 + '%0:s';
  ErrorParam = '   %s (%s)';

const
  BorderWidth = 8;
  UM_GetPropertyData = WM_USER + 2;
  UM_SetPropertyValue = WM_USER + 3;

  IntBool: array[boolean] of LongInt = (0, 1);
  IDVisible = 1;
  IdColor = 2;
  IdFontColor = 3;
  IdIcon = 4;
  IdIconIndex = 5;
  IdIconSize = 6;
  IdBoundsRect = 7;
  IdText = 8;
  IdAlphaBlendValue = 9;
  IdFont = 10;
  IdCanceled = 11;

type
  TPropertyData = packed record
    RecordSize: integer;
    DataSize: integer;
    Data: Pointer;
  end;
  PRopertyData = ^TPropertyData;

  TUMPropertyData = packed record
    Msg: Cardinal;
    IndexProp: Word;
    Part: Word;
    Value: PRopertyData;
    Result: Longint;
  end;

  TUMPropertyValue = packed record
    Msg: Cardinal;
    IndexProp: Word;
    Part: Word;
    Value: LongInt;
    Result: Longint;
  end;

  TElementPos = packed record
    TextRect: TRect;
    IconRect: TRect;
    ProgressRect: TRect;
    CaptionRect: TRect;
    ButtonRect: TRect;
    Reserv: TRect;
  end;

  TWndMethod = procedure(var Message: TMessage) of object;

  //     
  TCallDispath = (cdNone, cdOutThread, cdInThread, cdInWindow);
  TUpdateArea = (uaAll, //         
    uaWindow, //    
    uaIcon, //    
    uaProgress, //      
    uaButton,
    uaCaption,
    uaTimer);
  TUpdateAreas = set of TUpdateArea;

  TDefaultFont = (dCaptionFont, dSmCaptionFont, dMenuFont, dStatusFont, dMessageFont, dTimeFont);

  EThreadWindow = class(Exception)
  public
  end;

  TThreadWindow = class(TThread)
  private
    fBusy: THandle;
    fHandleCreated: THandle;
    fTextError: string;
    fErrorClass: ExceptClass;
    fVisible: boolean;
    fShowTick: DWORD;
    fHandleCreatedTick: DWORD;
    fTimer: UINT;
    fInterval: DWORD;
    fWND: HWND;
    fColor: Cardinal;
    fBrush: HBrush;
    fCaptionBrush: HBrush;
    fBoundsRect: TRect;
    fWindowRect: TRect;
    fClientRect: TRect;
    fHObj: Pointer;
    fCallDispath: TCallDispath;
    fLastMessage: TMessage;
    fFont: hFont;
    fFontColor: Cardinal;
    fIcon: HIcon;
    fIconSize: LongInt;
    fIconIndex: LongInt;
    fText: string;
    fCaption: string;
    fPercent: Integer;
    fOldPercent: SmallInt;
    fElementPos: TElementPos;
    fAlphaShow: boolean;
    fAlphaBlendValue: byte;
    fIntervalAlphaShow: integer;
    fEnabled: boolean;
    fOldEnabled: boolean;
    fUpdateAreas: TUpdateAreas;
    fHotPoint: TPoint;
    fCountInStack: integer;
    fLayoutUpdated: boolean;
    fStateButton: UINT;
    fCtl3D: Boolean;
    fOldCtl3d: Boolean;
    fCanceled: Boolean;
    fCaptionFont: HFont;
    fCaptionFontColor: Cardinal;
    fCaptionColor: Cardinal;
    fTimerFont: HFont;
    fTotalTime: TDateTime;
    fStartTime: TDateTime;
    fOldStartTime: TDateTime;
    fOldTimeText: string;
    fLowBlink: boolean;
    procedure AllocateHWnd(Method: TWndMethod);
    procedure DeallocateHWnd;
    procedure SetVisible(const Value: boolean);
    //  ProcessMessage      
    procedure ProcessMessage(MSG: TMSG);
    //  WNDProc      
    procedure WNDProc(var Message: TMessage);
    //  TimerProc      
    procedure TimerProc(var M: TMessage);
    //    
    procedure ShowWND;
    procedure CloseWND;
    procedure SetColor(const Value: Cardinal);
    procedure UpdateBrush(const Color: Cardinal; var Brush: HBrush);
    procedure SetBoundsRect(const Value: TRect);
    //          
    //       
    //     ,     ,
    //    
    procedure UMGetPropertyData(var Message: TUMPropertyData); message UM_GetPropertyData;
    //    .    ,  
    //   ,      perform
    procedure UMSetPropertyValue(var Message: TUMPropertyValue); message UM_SetPropertyValue;
    procedure WMSHOWWINDOW(var Message: TWMSHOWWINDOW); message WM_SHOWWINDOW;
    procedure WMWINDOWPOSCHANGED(var Message: TWMWINDOWPOSCHANGED); message WM_WINDOWPOSCHANGED;
    procedure WMERASEBKGND(var Message: TWMERASEBKGND); message WM_ERASEBKGND;
    procedure WMTIMER(var Message: TWMTIMER); message WM_TIMER;
    procedure WMLBUTTONDOWN(var Message: TWMLBUTTONDOWN); message WM_LBUTTONDOWN;
    procedure WMLBUTTONUP(var Message: TWMLBUTTONUP); message WM_LBUTTONUP;
    procedure SetFont(const Value: hFont);
    procedure SetFontColor(const Value: Cardinal);
    procedure SetIcon(const Value: HIcon);
    procedure FreeIcon(var DestIcon: HIcon);
    procedure UpdateIcon(SourceIcon: HIcon; var DestIcon: HIcon);
    procedure SetIconSize(const Value: LongInt);
    procedure SetText(const Value: string);
    procedure SetIconIndex(const Value: LongInt);
    procedure SetAlphaBlendValue(const Value: byte);
    procedure UpdateBlend(const WND: HWND; Value: Byte);
    function GetText: string;
    function GetVisible: boolean;
    function GetBoundsRect: TRect;
    function CalcTextRect(S: string; Font: HFont): TRect;
    procedure SetCanceled(const Value: boolean);
    procedure SetCaptionFont(const Value: HFont);
    function GetCaption: string;
    procedure SetCaption(const Value: string);
    procedure SetCaptionColor(const Value: Cardinal);
    procedure SetCaptionFontColor(const Value: Cardinal);
    procedure SetTimerFont(const Value: HFont);
    function GetTimeText: string;
  protected
    procedure Stop;
    procedure Execute; override;
    function MakeWParam(IndexProp, Part: Word): Longint; //inline;
    //    ,    
    procedure GetPropertyData(IndexProp, Part: Word; var Buffer; Size: LongInt); overload;
    procedure GetPropertyData(IndexProp, Part: Word; var S: string); overload;
    procedure GetPropertyData(IndexProp, Part: Word; var I: LongInt); overload;
    procedure GetPropertyData(IndexProp, Part: Word; var B: boolean); overload;
    //      - ,   
    //   
    procedure DoTerminate; override;
    //  DoBeforeResume        
    procedure DoBeforeResume; virtual;
    //  DoAfterStop        
    procedure DoAfterStop; virtual;
    //  WNDCreated    
    procedure WNDCreated(WND: HWND); virtual;
    //  WNDDestroy    
    procedure WNDDestroy(var WND: HWND); virtual;
    //  DoBeforeDispath    
    procedure DoBeforeDispath(var Message: TMessage); virtual;
    //  DrawWND    
    procedure DrawWND(DC: HDC; ARect: TRect); virtual;
    //  DrawProgress     
    procedure DrawProgress(DC: HDC; ARect: TRect; Percent: integer); virtual;
      //     
    procedure DrawButton(DC: HDC; ARect: TRect; State: UINT); virtual;
      //  DrawTime   
    procedure DrawTime(DC: HDC; ARect: TRect; StartTime, TotalTime: TDateTime); virtual;
    //  DoAfterSetBounds    
    //         
    procedure DoAfterSetBounds(NewBoundsRect: TRect; var NewWindowRect, NewClientRect: TRect); virtual;
    //  UpdateLayout     
    procedure UpdateLayout(ClientRect: TRect; var ElementPos: TElementPos); virtual;
    //  DrawText    
    procedure DrawText(DC: HDC; ARect: TRect; Text: PChar; Len: integer); virtual;
    //  DrawCaption   
    procedure DrawCaption(DC: HDC; ARect: TRect; Text: PChar; Len: integer); virtual;
    //  DrawIcon   
    procedure DrawIcon(DC: HDC; ARect: TRect; Icon: HIcon; Brush: HBrush); virtual;
    //     
    procedure DrawBorder(DC: HDC; ARect: TRect; Ctl3D: boolean; Width: Integer;
      BorderColor: Cardinal;
      IsDown: boolean = false); virtual;
    //  
    procedure DrawBackground(DC: HDC; ARect: TRect; Ctl3D: boolean); virtual;
    // ,     fBusy.
    //  ,     
    //  SetAfterWaiting = true,      
    //    ( ),   .
    //   ,   ,   false,  true
    function WaitBusy(SetAfterWaiting: boolean = false): boolean;
    //       
    // (     > WM_USER)
    function Perform(Msg: Cardinal; WParam, LParam: Longint): Longint;
    //  Change ,  ,        
    //  
    procedure Change(var UpdateAreas: TUpdateAreas); virtual;
    //    ,      
    procedure Invalidate(Area: TUpdateArea = uaWindow);
    //    True,    
    // ,    .
    function InThread: boolean;
    function DefaultBoundsRect: TRect; virtual;
    property LowBlink: boolean read fLowBlink write fLowBlink;
  public
    //        
    //    AMessage,    ,
    //    DefaultMessage
    constructor Show(AMessage: string = ''); overload;
    //   
    destructor Destroy; override;
    //       
    procedure AfterConstruction; override;
    //       Font,    0, 
    //       
    class function GetFontIndirect(Font: HFont; DefaultFont: TDefaultFont): tagLOGFONT;
    //   
    //            
    //   .    
    // GetVisible  GetText  
    property Visible: boolean read GetVisible write SetVisible;
    //  
    property Color: Cardinal read fColor write SetColor;
    property CaptionColor: Cardinal read fCaptionColor write SetCaptionColor;
      //       ,  
    property Ctl3D: Boolean read fCtl3D write fCtl3D;
      //        ""
      //      
    property Canceled: boolean read fCanceled write SetCanceled;
    //   ,   .     ,   
    // c   . . WindowRect
    property BoundsRect: TRect read GetBoundsRect write SetBoundsRect;
      //   
    property Font: hFont read fFont write SetFont;
      //   
    property CaptionFont: HFont read fCaptionFont write SetCaptionFont;
      //     
    property TimerFont: HFont read fTimerFont write SetTimerFont;


    property FontColor: Cardinal read fFontColor write SetFontColor;
    property CaptionFontColor: Cardinal read fCaptionFontColor write SetCaptionFontColor;
    property Icon: HIcon read fIcon write SetIcon;
    property IconIndex: LongInt read fIconIndex write SetIconIndex;
    property IconSize: LongInt read fIconSize write SetIconSize;
    //      .
    //  0,   Visible  False
    property AlphaBlendValue: byte read fAlphaBlendValue write SetAlphaBlendValue;
    //  
    property Text: string read GetText write SetText;
    property Message: string read GetText write SetText;
      // .    ,    
    property Caption: string read GetCaption write SetCaption;
      //      .  
      //    Now.
    property StartTime: TDateTime read fStartTime write fStartTime;
      //       ( )
      //     0,    
      //  
    property TotalTime: TDateTime read fTotalTime write fTotalTime;

    //  .
    //        .   Change
    //      ,    
    //   ,     
    //  
    property WND: HWND read fWND;
    //     
    property Brush: HBrush read fBrush;
    //     ()
    property Interval: DWORD read fInterval write fInterval;
    //   GetTickCount     Visible
    property ShowTick: DWORD read fShowTick;
    //     10 (0..1000).   0,   
    property Percent: Integer read fPercent write fPercent;
    //   ,   
    property WindowRect: TRect read fWindowRect;
    //        
    //   
    property ClientRect: TRect read fClientRect;
    //  CallDispath   ,    
    //  
    property CallDispath: TCallDispath read fCallDispath;
    //    ,     
    //   .  ,   
    //   
    property AlphaShow: boolean read fAlphaShow write fAlphaShow;
    //        
    property IntervalAlphaShow: integer read fIntervalAlphaShow write fIntervalAlphaShow;
    //   (     ..)
    property Enabled: boolean read fEnabled write fEnabled;
  end;

function Test(AllMembers, Member: Cardinal): boolean;
function Blend(Color1, Color2: Cardinal; BlendColor1: Byte): Cardinal;

implementation
...
