unit uEditActionTreeView;

interface
{$I Directives.inc}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ActionTreeView, ExtCtrls,
  ActnList,
  {$IFDEF DELPHI_6_UP}
  ActnMan, GraphUtil,
  {$ENDIF}
  ImgList,
  ImageListCep, InternalTimer, ToolWin, SpeedButtonCep, Mask, Menus;
const
  CenterWidth = 8;
type
  TSourceActionTreeView = class (TActionTreeView)
  protected
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    function NodeClientsItemClass: TNodeClientClass; override;
    function ActionNodeClass: TActionNodeClass; override;
    procedure KeyPress(var Key: Char); override;
    procedure DblClick; override;
  end;

  TSourceNodeClient = class (TNodeClient)
  private
    fStateIndex: integer;
    fIsFound: boolean;
  protected
    function ItemClass: TNodeClientClass; override;
  public
    property StateIndex: integer read fStateIndex write fStateIndex;
    property IsFound: boolean read fIsFound write fIsFound;
  end;

  TSourceActionNode = class (TActionNode)
  private
  public
    procedure DrawImage(TreeView: TActionTreeView; State: TCustomDrawState;
      Image: TCustomImageList; Index: integer; R: TRect); override;
  end;

  TResultActionTreeView = class (TActionTreeView)
  private
    fIntTimer: TInternalTimer;
    fOnClientsChanged: TNotifyEvent;
    procedure DoDelayChanged;
  protected
    procedure IntTimerProc(Sender: TObject);
    function NodeClientsItemClass: TNodeClientClass; override;
    function ActionNodeClass: TActionNodeClass; override;
    procedure KeyPress(var Key: Char); override;
    procedure DblClick; override;
  public
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property OnClientsChanged: TNotifyEvent read fOnClientsChanged write fOnClientsChanged;
  end;

  TResultNodeClient = class (TNodeClient)
  private
    fStateIndex: integer;
  protected
    procedure AssignValues(Source: TPersistent); override;
    function ItemClass: TNodeClientClass; override;
  public
    property StateIndex: integer read fStateIndex write fStateIndex;
  end;

  TResultActionNode = class (TActionNode)
  private
  public
    procedure DrawImage(TreeView: TActionTreeView; State: TCustomDrawState;
      Image: TCustomImageList; Index: integer; R: TRect); override;
  end;

  TImageNumber = record
    Images: TCustomImageList;
    CurrIndex, AllIndex: integer;
    Added: boolean;
    Action: TCustomAction;
  end;

  TEditActionTreeView = class(TForm)
    pTop: TPanel;
    pBottom: TPanel;
    StateImages: TImageListCep;
    ButtonImages: TImageListCep;
    ActionList1: TActionList;
    actAdd: TAction;
    actAddChild: TAction;
    actDelSingle: TAction;
    actDelAll: TAction;
    actEnabled: TAction;
    actHide: TAction;
    actExpanded: TAction;
    actChecked: TAction;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    meCaption: TMaskEdit;
    pToolBar: TPanel;
    actCopyFromAction: TAction;
    ToolButton10: TToolButton;
    actCopyToAction: TAction;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    Button1: TButton;
    Button2: TButton;
    cbImages: TComboBox;
    cbImageIndex: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure actAddExecute(Sender: TObject);
    procedure actAddChildExecute(Sender: TObject);
    procedure actDelSingleExecute(Sender: TObject);
    procedure actDelAllExecute(Sender: TObject);
    procedure actEnabledExecute(Sender: TObject);
    procedure actHideExecute(Sender: TObject);
    procedure actExpandedExecute(Sender: TObject);
    procedure actAddUpdate(Sender: TObject);
    procedure actDestNeed(Sender: TObject);
    procedure actEnabledUpdate(Sender: TObject);
    procedure actHideUpdate(Sender: TObject);
    procedure actExpandedUpdate(Sender: TObject);
    procedure actCheckedExecute(Sender: TObject);
    procedure actCheckedUpdate(Sender: TObject);
    procedure meCaptionChange(Sender: TObject);
    procedure actCopyFromActionExecute(Sender: TObject);
    procedure actCopyToActionExecute(Sender: TObject);
    procedure actCopyFromActionUpdate(Sender: TObject);
    procedure actCopyToActionUpdate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure cbImagesChange(Sender: TObject);
    procedure cbImageIndexMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure cbImageIndexDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure cbImageIndexChange(Sender: TObject);
  private
    { Private declarations }
    fTreeSource: TSourceActionTreeView;
    fTreeResult: TResultActionTreeView;
    fEnableActionEdit: boolean;
    fAllResultImages: TImageListCep;
    fSourceImageNumbers: array of TImageNumber;
    procedure AllignTop;
    procedure SetActionTreeView(const Value: TActionTreeView);
    function GetTreeResult: TActionTreeView;
    function DestNodeClient: TNodeClient;
    procedure SetEnableActionEdit(const Value: boolean);
    function DestNodeAction: TCustomAction;
    procedure UpdateEditEnabled;
    procedure UpdateSourceImages;
    function ImagesEqual: boolean;
    procedure ResultImagesChange(Sender: TObject);
  protected
    procedure Resize; override;
    procedure CreateWnd; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddSourceActions(List: TStringList);
    procedure AddImages(List: TStringList);
    procedure UpdateSourceNodes(Sender: TObject);
    procedure ResultChange(Sender: TObject; Node: TTreeNode);
    property ActionTreeView: TActionTreeView read GetTreeResult write SetActionTreeView;
    property EnableActionEdit: boolean read fEnableActionEdit write SetEnableActionEdit;
  end;

var EditActionTreeView: TEditActionTreeView;

implementation
...
